package nonvisualdemo;

import com.adbs.ast.*;
import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

public class MainFrame extends javax.swing.JFrame
{
	private static final long serialVersionUID = 1L;

	QueryBuilder queryBuilder = new QueryBuilder(true); // create querty builder in non-visual mode (true paramerter);

    public MainFrame()
	{
        initComponents();

		// center window on the desktop
		setLocationRelativeTo(null);

		queryBuilder.setSyntaxProvider(new MSSQLSyntaxProvider());

		textPane.setText("SELECT SUM(o.Payment) AS Total, c.Name, c.Address\n" +
"FROM MyDB.MySchema.Orders o\n" +
"    INNER JOIN MyDB.MySchema.Customers c ON o.CustomerID = c.CustomerID\n" +
"        LEFT JOIN MyDB.MySchema.OrderDetails od ON od.OrderID = o.OrderID\n" +
"WHERE\n" +
"    o.Special = 1\n" +
"    OR\n" +
"        o.CustomerID = c.CustomerID\n" +
"        AND\n" +
"        o.OrderID > 0\n" +
"GROUP BY o.CustomerID, c.Name, c.Address\n" +
"HAVING SUM(o.Payment) > 1000\n" +
"ORDER BY Total DESC, c.Name");

		fillMetadataContainer();
    }

	// HOWTO: Fill metadata container with custom objects
	private void fillMetadataContainer()
	{
		MetadataContainer mc = queryBuilder.getMetadataContainer();

		// create table
		MetadataObject tableOrders = mc.addTable("MySchema", "Orders", false, "MyDB");
		tableOrders.getFields().addField("OrderID");
		tableOrders.getFields().addField("OrderDate");
		tableOrders.getFields().addField("CustomerID");
		tableOrders.getFields().addField("ResellerID");

		// create another table
		MetadataObject tableCustomers = mc.addTable("MySchema", "Customers", false, "MyDB");
		tableCustomers.getFields().addField("CustomerID");
		tableCustomers.getFields().addField("CustomerName");
		tableCustomers.getFields().addField("CustomerAddress");

		// add a relation between these two tables
		MetadataRelation relation = tableCustomers.getRelations().add();
		relation.setChildSchemaStr("MySchema");
		relation.setChildDatabaseStr("MyDB");
		relation.setChildNameStr("Orders");
		relation.getKeyFields().addField("CustomerID");
		relation.getChildFields().addField("CustomerID");

		//create view
		MetadataObject viewResellers = mc.addView("MySchema", "Resellers", false, "MyDB");
		viewResellers.getFields().addField("ResellerID");
		viewResellers.getFields().addField("ResellerName");
	}

	// HOWTO: Create a query programmatically
	public void createQuery()
	{
		UnionSubQuery sq = (UnionSubQuery) queryBuilder.getQuery().get(0);

		// add data source objects into the query
		// AddObjectWithFK() analyzes object relations and they will be automatically used in our query
		DataSource ds1 = sq.addObjectWithFK("MyDB.MySchema.Customers", "c");
		DataSource ds2 = sq.addObjectWithFK("MyDB.MySchema.Orders", "o");
		DataSource ds3 = sq.addObjectWithFK("MyDB.MySchema.Resellers", "r");

		// manually create relation between 'Resellers' and 'Orders'
		// this relation doesn't exist in metadata structure we filled, but we need it for our query
		sq.addLink(ds3, "ResellerID", ds2, "ResellerID");

		// create first output column (expression)
		CriteriaItem ci1 = sq.getCriteriaList().addExpression("GetDate()");
		ci1.setSelected(true);
		ci1.setAliasString("d");

		// create simple WHERE condition
		CriteriaItem ci2 = sq.getCriteriaList().addExpression("CustomerID");
		ci2.setSelected(false);
		ci2.setConditionString(0, "> 0");

		// create second output column
		CriteriaItem ci3 = sq.getCriteriaList().addField(ds1, "CustomerName");
		ci3.setSelected(true);

		// create third output column with condition
		CriteriaItem ci4 = sq.getCriteriaList().addField(ds2, "OrderID");
		ci4.setSelected(true);
		ci4.setConditionString(0, "> 0");

		// order by...
		ci3.setSortType(ItemSortType.Desc);
		ci3.setSortOrder(0);

		// at this stage you can get simple unformatted query text...
		//textBox1.Text = qb.SQL;

		// ... or format the query text with SQL formatter
		PlainTextSQLBuilder sb = new PlainTextSQLBuilder();
		sb.setQueryBuilder(queryBuilder);
		sb.setKeywordFormat(KeywordFormat.UpperCase);

		// send the result SQL query text to the text box
		textPane.setText(sb.getSQL());
	}

	// HOWTO: get objects from metadata container
	public void analyzeMetadataContainerContent()
	{
		MetadataContainer mc = queryBuilder.getMetadataContainer();
		MetadataObject mo;
		StringBuilder s;
		StringBuilder tables = new StringBuilder("Tables:\r\n");
		StringBuilder views = new StringBuilder("Views:\r\n");

		for (int i = 0; i < mc.getCount(); i++)
		{
			mo = mc.get(i);

			if (mo instanceof MetadataTable)
			{
				s = tables;
			}
			else if (mo instanceof MetadataView)
			{
				s = views;
			}
			else
			{
				s = null; // process only tables and views
			}

			if (s != null)
			{
				// append object name
				s.append("    " + mo.getFullNameStr() + "\r\n");

				MetadataFieldList fields = mo.getFields();

				// append fields
				for (int j = 0; j < fields.getCount(); j++)
				{
					// append field name
					s.append("        " + fields.get(j).getNameStr());

					MetadataRelationList relations = mo.getRelations();

					// search relations for this field
					for (int y = 0; y < relations.getCount(); y++)
					{
						MetadataQualifiedNameList keyFields = relations.get(y).getKeyFields();

						for (int z = 0; z < keyFields.getCount(); z++)
						{
							if (keyFields.get(z).getQualifiedName().equals(fields.get(j).getNameStr()))
							{
								s.append(" -- > " + relations.get(y).getChildTableStr() + "." +
										relations.get(y).getChildFields().get(z).getQualifiedName());
							}
						}
					}

					s.append("\r\n");
				}
			}
		}

		JOptionPane.showMessageDialog(this, tables.toString() + "\r\n" + views.toString());
	}

	public void analyzeWhereClause()
	{
		try
		{
			queryBuilder.setSQL(textPane.getText());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage(), "Parsing error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		// find a union-subquery to analyze
		SQLSubQuerySelectExpression select = findFirstSelect(queryBuilder.getResultQueryAST());

		if (select.where != null)
		{
			// create form to display results
			QueryStructureDialog dlg = new QueryStructureDialog(this, true);

			dlg.setTitle("WHERE structure");
			DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");
			DefaultTreeModel model = new DefaultTreeModel(root);
			loadCondition(root, select.where);

			dlg.jTree1.setRootVisible(false);
			dlg.jTree1.setModel(model);
			dlg.jTree1.expandPath(new TreePath(root.getPath()));

			// expand all nodes
			for (int i = 0; i < dlg.jTree1.getRowCount(); i++)
			{
				dlg.jTree1.expandRow(i);
			}

			dlg.setVisible(true);
		}
	}

	// HOWTO: Get query statistics
	public void queryStatistics()
	{
		try
		{
			queryBuilder.setSQL(textPane.getText());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage(), "Parsing error", JOptionPane.ERROR_MESSAGE);
			return;
		}


		String stats = "";

		QueryStatistics qs = queryBuilder.getQueryStatistics();

		StatisticsDatabaseObjectList usedDatabaseObjects = qs.getUsedDatabaseObjects();

		stats = "Used Objects (" + usedDatabaseObjects.getCount() + "):\r\n";
		for (int i = 0; i < usedDatabaseObjects.getCount(); i++)
		{
			stats += "\r\n" + usedDatabaseObjects.get(i).getFullName().getQualifiedName();
		}

		StatisticsColumnList usedDatabaseObjectColumns = qs.getUsedDatabaseObjectColumns();

		stats += "\r\n\r\n" + "Used Columns (" + usedDatabaseObjectColumns.getCount() + "):\r\n";
		for (int i = 0; i < usedDatabaseObjectColumns.getCount(); i++)
		{
			stats += "\r\n" + usedDatabaseObjectColumns.get(i).getFullName().getQualifiedName();
		}

		StatisticsOutputColumnList outputColumns = qs.getOutputColumns();

		stats += "\r\n\r\n" + "Output Expressions (" + outputColumns.getCount() + "):\r\n";
		for (int i = 0; i < outputColumns.getCount(); i++)
		{
			stats += "\r\n" + outputColumns.get(i).getExpression();
		}

		QueryStatisticsDialog dlg = new QueryStatisticsDialog(this, true);

		dlg.jTextPane1.setText(stats);
		dlg.setVisible(true);

		//Notice:
		// You also can use the query statistics to get the query structure in XML form:
		//String structure = queryBuilder.getQueryStatistics().getQueryStructureXML();
	}

	// HOWTO: Analyze a query containing in the query builder
	//		  Fill a sample tree view with query components
	public void analyzeQuery()
	{
		try
		{
			queryBuilder.setSQL(textPane.getText());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage(), "Parsing error", JOptionPane.ERROR_MESSAGE);
			return;
		}

		// if you want to hide default database and schema from object names, uncomment lines below
		//queryBuilder.getMetadataContainer().setDefaultDatabaseNameStr("MyDB");
		//queryBuilder.getMetadataContainer().setDefaultSchemaNamesStr("MySchema");

		QueryStructureDialog dlg = new QueryStructureDialog(this, true);

		dlg.setTitle("Query structure");
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("root");
		DefaultTreeModel model = new DefaultTreeModel(root);

		// fill the tree model with query structure

		// Note: if you don't need to analyze the whole query structure
        // with all sub-queries and unions, you may simply call use the
        // queryBuilder.getQuery().getFirstSelect() to get access to the main query.
        // You may uncomment the next line and comment the line aftet that
        // to load information about main query only.

		//loadUnionSubQuery(root, queryBuilder.getQuery().getFirstSelect());
		loadQuery(root, queryBuilder.getQuery());

		dlg.jTree1.setRootVisible(false);
		dlg.jTree1.setModel(model);
		dlg.jTree1.expandPath(new TreePath(root.getPath()));

		// expand all nodes
		for (int i = 0; i < dlg.jTree1.getRowCount(); i++)
		{
			dlg.jTree1.expandRow(i);
		}

		dlg.setVisible(true);
	}

	private DefaultMutableTreeNode loadQuery(DefaultMutableTreeNode parent, SubQuery q)
	{
		DefaultMutableTreeNode result = addTreeNode(parent, q.getCaption());

		// union parts
		if (q.getCount() == 1 && q.get(0) instanceof UnionSubQuery)
		{
			loadUnionSubQuery(result, (UnionSubQuery) q.get(0));
		}
		else
		{
			loadUnionGroup(result, q, "Unions");
		}

		// subqueries
		for (int i = 0; i < q.getSubQueryCount(); i++)
		{
			loadQuery(result, (SubQuery) q.getSubQuery(i));
		}

		return result;
	}

	private void loadUnionGroup(DefaultMutableTreeNode parent, UnionGroup unionGroup, String caption)
	{
		DefaultMutableTreeNode unionNode = addTreeNode(parent, caption);

		for (int i = 0; i < unionGroup.getCount(); i++)
		{
			if (unionGroup.get(i) instanceof UnionGroup)
			{
				loadUnionGroup(unionNode, (UnionGroup) unionGroup.get(i), "( )");
			}
			else if (unionGroup.get(i) instanceof UnionSubQuery)
			{
				UnionSubQuery unionSubQuery = (UnionSubQuery) unionGroup.get(i);
				DefaultMutableTreeNode subqueryNode = addTreeNode(unionNode, unionSubQuery.getResultSQL(
						unionSubQuery.getSQLContext().getSQLBuilderExpressionForServer()));
				loadUnionSubQuery(subqueryNode, unionSubQuery);
			}
		}
	}

	private void loadUnionSubQuery(DefaultMutableTreeNode parent, UnionSubQuery unionSubQuery)
	{
		// Notice: At this stage you can retrieve whole sub-query or it's different parts in string form:
		// whole sub-query:
//		String unionSubqueryString = unionSubQuery.getResultSQL();
		// parts:
//		String selectListString = unionSubQuery.getSelectListString();
//		String fromString = unionSubQuery.getFromClauseString();
//		String whereString = unionSubQuery.getWhereClauseString();
//		String groupByString = unionSubQuery.getGroupByClauseString();
//		String havingString = unionSubQuery.getHavingClauseString();
//		String orderByString = unionSubQuery.getOrderByClauseString();

		DefaultMutableTreeNode listNode = addTreeNode(parent, "Fields");
		// Get structure of union subquery components
		// SELECT list

		CriteriaList criteriaList = unionSubQuery.getCriteriaList();

		for (int i = 0; i < criteriaList.getCount(); i++)
		{
			CriteriaItem criteriaItem = criteriaList.get(i);

			if (criteriaItem.isSelected())
			{
				String s = criteriaItem.getExpressionString();

				if (criteriaItem.getAggregate() != null)
				{
					s = criteriaItem.getAggregateString() + "(" + s + ")";
				}

				if (!isNullOrEmpty(criteriaItem.getAliasString()))
				{
					s += " " + criteriaItem.getAliasString();
				}

				addTreeNode(listNode, s);
			}
		}

		// FROM clause

		if (unionSubQuery.getFromClause().getCount() > 0)
		{
			listNode = addTreeNode(parent, "FROM");

			loadFromGroup(listNode, unionSubQuery.getFromClause());
		}

		// WHERE clause

		// compute ORs count
		int orsCount = 0;

		for (int i = 0; i < criteriaList.getCount(); i++)
		{
			orsCount = Math.max(criteriaList.get(i).getConditionCount(), orsCount);
		}

		// collect conditions
		if (orsCount > 0)
		{
			DefaultMutableTreeNode whereNode = addTreeNode(parent, "WHERE");

			for (int or = 0; or < orsCount; or++)
			{
				// new OR item
				DefaultMutableTreeNode orNode = addTreeNode(whereNode, "OR");

				for (int i = 0; i < criteriaList.getCount(); i++)
				{
					CriteriaItem criteriaItem = criteriaList.get(i);

					if (criteriaItem.getConditionType() == ConditionType.Where && !isNullOrEmpty(criteriaItem.getConditionString(or)))
					{
						// add AND item
						 addTreeNode(orNode, "AND " + criteriaItem.getExpressionString() + " " +
								 criteriaItem.getConditionString(or));
					}
				}
			}
		}

		// GROUP BY clause

		// compute GROUPBYs count
		int groupbyCount = 0;

		for (int i = 0; i < criteriaList.getCount(); i++)
		{
			if (criteriaList.get(i).isGrouping())
			{
				groupbyCount++;
			}
		}

		if (groupbyCount > 0)
		{
			DefaultMutableTreeNode groupbyNode =  addTreeNode(parent, "GROUP BY");

			for (int i = 0; i < criteriaList.getCount(); i++)
			{
				if (criteriaList.get(i).isGrouping())
				{
					addTreeNode(groupbyNode, criteriaList.get(i).getExpressionString());
				}
			}
		}

		// HAVING clause
		int havingsCount = 0;

		for (int or = 0; or < orsCount; or++)
		{
			for (int i = 0; i < criteriaList.getCount(); i++)
			{
				CriteriaItem criteriaItem = criteriaList.get(i);

				if (criteriaItem.getConditionType() == ConditionType.Having &&
						!isNullOrEmpty(criteriaItem.getConditionString(or)))
				{
					havingsCount++;
				}
			}
		}

		if (havingsCount > 0)
		{
			DefaultMutableTreeNode havingNode =  addTreeNode(parent, "HAVING");

			for (int or = 0; or < orsCount; or++)
			{
				// new OR item
				DefaultMutableTreeNode orNode = new DefaultMutableTreeNode("OR");

				for (int i = 0; i < criteriaList.getCount(); i++)
				{
					CriteriaItem criteriaItem = criteriaList.get(i);

					if (criteriaItem.getConditionType() == ConditionType.Having &&
							!isNullOrEmpty(criteriaItem.getConditionString(or)))
					{
						// add AND item
						addTreeNode(orNode, "AND " + criteriaItem.getExpressionString() + " " +
								criteriaItem.getConditionString(or));
					}
				}

				if (orNode.getChildCount() > 0)
				{
					havingNode.add(orNode);
				}
			}
		}

		// ORDER BY clause

		// collect ORDERBYs items

		ArrayList<CriteriaItem> orderbyList = new ArrayList<CriteriaItem>();

		for (int i = 0; i < criteriaList.getCount(); i++)
		{
			if (!isNullOrEmpty(criteriaList.get(i).getSortOrderString()))
			{
				orderbyList.add(criteriaList.get(i));
			}
		}

		if (orderbyList.size() > 0)
		{
			// sort ORDER BY items by theirs index
			Collections.sort(orderbyList, new Comparator<CriteriaItem>()
			{
				public int compare(CriteriaItem o1, CriteriaItem o2)
				{
					if (o1.getSortOrder() > o2.getSortOrder())
					{
						return 1;
					}

					if (o1.getSortOrder() < o2.getSortOrder())
					{
						return -1;
					}

					return 0;
				}
			});

			//orderbyList.sort(CompareOrderByItems);

			DefaultMutableTreeNode orderbyNode = addTreeNode(parent, "ORDER BY");

			for (int i = 0; i < orderbyList.size(); i++)
			{
				String s = orderbyList.get(i).getExpressionString() + " " + orderbyList.get(i).getSortTypeString();
				addTreeNode(orderbyNode, s);
			}
		}
	}

	private void loadFromGroup(DefaultMutableTreeNode parent, DatasourceGroup f)
	{
		for (int i = 0; i < f.getCount(); i++)
		{
			DataSourceBase ds = f.get(i);

			if (ds instanceof DatasourceGroup)
			{
				loadFromGroup(parent, (DatasourceGroup) ds);
			}
			else
			{
				addTreeNode(parent, ds.getResultSQL(ds.getSQLContext().getSQLBuilderExpressionForServer()));
			}
		}

		if (f instanceof DatasourceGroup) // FROM clause
		{
			if (f.getLinkCount() > 0)
			{
				DefaultMutableTreeNode linksNode = addTreeNode(parent, "LINKS");

				for (int i = 0; i < f.getLinkCount(); i++)
				{
					String join;

					if (f.links.get(i).getLeftType() == LinkSideType.Inner)
					{
						if (f.links.get(i).getRightType() == LinkSideType.Inner)
						{
							join = "INNER JOIN";
						}
						else
						{
							join = "RIGHT JOIN";
						}
					}
					else if (f.links.get(i).getRightType() == LinkSideType.Inner)
					{
						join = "LEFT JOIN";
					}
					else
					{
						join = "FULL JOIN";
					}

					join += " " + f.links.get(i).getLinkExpression().getSimpleSQL(
							f.getSQLContext().getSQLBuilderExpressionForServer());

					addTreeNode(linksNode, join);
				}
			}
		}
	}

	private SQLSubQuerySelectExpression findFirstSelect(SQLSubQueryExpressions queryGroup)
	{
		SQLSubQuerySelectExpression result = null;

		for (int i = 0; i < queryGroup.getCount(); i++)
		{
			if (queryGroup.get(i) instanceof SQLSubQuerySelectExpression)
			{
				return (SQLSubQuerySelectExpression) queryGroup.get(i);
			}
			else if (queryGroup.get(i) instanceof SQLSubQueryExpressions)
			{
				result = findFirstSelect((SQLSubQueryExpressions) queryGroup.get(i));

				if (result != null)
				{
					return result;
				}
			}
		}

		return result;
	}

	private DefaultMutableTreeNode loadCondition(DefaultMutableTreeNode parent, SQLExpressionItem condition)
	{
		DefaultMutableTreeNode result = null;
		String s;

		if (condition == null)
		{
			// NULL reference protection
			result = addTreeNode(parent, "--null--");
		}
		else if (condition instanceof SQLExpressionBrackets)
		{
			// The condition is actually the "Brackets" structural node.
			// Create the "brackets" tree node and load the brackets content as children tree nodes.
			result = addTreeNode(parent, "( )");
			loadCondition(result, ((SQLExpressionBrackets) condition).lExpression);
		}
		else if (condition instanceof SQLExpressionOr)
		{
			// The condition is actually the "OR Collection" structural node.
			// Create the "OR" tree node and load all items of the collection as children tree nodes.
			result = addTreeNode(parent, "OR");

			for (int i = 0; i < ((SQLExpressionOr) condition).getCount(); i++)
			{
				loadCondition(result, ((SQLExpressionOr) condition).get(i));
			}
		}
		else if (condition instanceof SQLExpressionAnd)
		{
			// The condition is actually the "AND Collection" structural node.
			// Create the "AND" tree node and load all items of the collection as children tree nodes.
			result = addTreeNode(parent, "AND");

			for (int i = 0; i < ((SQLExpressionAnd) condition).getCount(); i++)
			{
				loadCondition(result, ((SQLExpressionAnd) condition).get(i));
			}
		}
		else if (condition instanceof SQLExpressionNot)
		{
			// The condition is actually the "NOT" structural node.
			// Create the "NOT" tree node and load content of the "NOT" operator as children tree nodes.
			result = addTreeNode(parent, "NOT");
			loadCondition(result, ((SQLExpressionNot) condition).lExpression);
		}
		else if (condition instanceof SQLExpressionOperatorBinary)
		{
			// The condition is actually the "Binary Operator" structural node.
			// Create a tree node containing the operator value and two leaf nodes with the operator arguments.
			s = ((SQLExpressionOperatorBinary) condition).operatorObj.operatorName;
			result = addTreeNode(parent, s);
			// left argument of the binary operator
			loadCondition(result, ((SQLExpressionOperatorBinary) condition).lExpression);
			// right argument of the binary operator
			loadCondition(result, ((SQLExpressionOperatorBinary) condition).rExpression);
		}
		else
		{
			// Other types of structureal nodes.
			// Just show them as a text.
			s = condition.getSimpleSQL(condition.getSQLContext().getSQLBuilderExpression());
			result = addTreeNode(parent, s);
		}

		return result;
	}

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        cmbSyntax = new javax.swing.JComboBox();
        jScrollPane1 = new javax.swing.JScrollPane();
        textPane = new javax.swing.JTextPane();
        jLabel2 = new javax.swing.JLabel();
        btnAnalyzeQuery = new javax.swing.JButton();
        btnQueryStatistics = new javax.swing.JButton();
        btnAnalyzeMetadataContainer = new javax.swing.JButton();
        btnAnalyzeWhereClause = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Non-visual use demo");

        jLabel1.setText("Syntax:");

        cmbSyntax.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "ANSI SQL-2003", "ANSI SQL-89", "ANSI SQL-92", "Firebird", "IBM DB2", "IBM Informix", "MS Access", "MS SQL Server", "MySQL", "Oracle", "PostgreSQL", "SQLite", "Sybase" }));
        cmbSyntax.setSelectedIndex(7);
        cmbSyntax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cmbSyntaxActionPerformed(evt);
            }
        });

        jScrollPane1.setViewportView(textPane);

        jLabel2.setText("Query:");

        btnAnalyzeQuery.setText("Analyze query");
        btnAnalyzeQuery.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAnalyzeQueryActionPerformed(evt);
            }
        });

        btnQueryStatistics.setText("Query statistics");
        btnQueryStatistics.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnQueryStatisticsActionPerformed(evt);
            }
        });

        btnAnalyzeMetadataContainer.setText("Analyze metadata container");
        btnAnalyzeMetadataContainer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAnalyzeMetadataContainerActionPerformed(evt);
            }
        });

        btnAnalyzeWhereClause.setText("Analyze WHERE clause");
        btnAnalyzeWhereClause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAnalyzeWhereClauseActionPerformed(evt);
            }
        });

        jScrollPane2.setBorder(null);
        jScrollPane2.setFont(new java.awt.Font("Tahoma", 1, 11));

        jTextArea1.setBackground(java.awt.SystemColor.control);
        jTextArea1.setColumns(20);
        jTextArea1.setFont(new java.awt.Font("Tahoma", 1, 11));
        jTextArea1.setRows(5);
        jTextArea1.setText("See the source code for:\n- how to analyze a query\n- how to fill the metadata container with custom objects\n- how to create a query programmatically\n- how to get objects from the metadata container");
        jScrollPane2.setViewportView(jTextArea1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 540, Short.MAX_VALUE)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 540, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(btnAnalyzeQuery, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(btnAnalyzeWhereClause, javax.swing.GroupLayout.DEFAULT_SIZE, 157, Short.MAX_VALUE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnQueryStatistics, javax.swing.GroupLayout.PREFERRED_SIZE, 150, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnAnalyzeMetadataContainer, javax.swing.GroupLayout.PREFERRED_SIZE, 219, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(cmbSyntax, javax.swing.GroupLayout.PREFERRED_SIZE, 161, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(cmbSyntax, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel2)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 195, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnAnalyzeQuery, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnAnalyzeMetadataContainer, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnQueryStatistics, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnAnalyzeWhereClause, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 97, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void cmbSyntaxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cmbSyntaxActionPerformed
	{//GEN-HEADEREND:event_cmbSyntaxActionPerformed
		String syntax = (String) cmbSyntax.getSelectedItem();

		if (syntax.equals("ANSI SQL-2003"))
		{
			queryBuilder.setSyntaxProvider(new SQL2003SyntaxProvider());
		}
		else if (syntax.equals("ANSI SQL-89"))
		{
			queryBuilder.setSyntaxProvider(new SQL89SyntaxProvider());
		}
		else if (syntax.equals("ANSI SQL-92"))
		{
			queryBuilder.setSyntaxProvider(new SQL92SyntaxProvider());
		}
		else if (syntax.equals("Firebird"))
		{
			queryBuilder.setSyntaxProvider(new FirebirdSyntaxProvider());
		}
		else if (syntax.equals("IBM DB2"))
		{
			queryBuilder.setSyntaxProvider(new DB2SyntaxProvider());
		}
		else if (syntax.equals("IBM Informix"))
		{
			queryBuilder.setSyntaxProvider(new InformixSyntaxProvider());
		}
		else if (syntax.equals("MS Access"))
		{
			queryBuilder.setSyntaxProvider(new AccessSyntaxProvider());
		}
		else if (syntax.equals("MS SQL Server"))
		{
			queryBuilder.setSyntaxProvider(new MSSQLSyntaxProvider());
		}
		else if (syntax.equals("MySQL"))
		{
			queryBuilder.setSyntaxProvider(new MySQLSyntaxProvider());
		}
		else if (syntax.equals("Oracle"))
		{
			queryBuilder.setSyntaxProvider(new OracleSyntaxProvider());
		}
		else if (syntax.equals("PostgreSQL"))
		{
			queryBuilder.setSyntaxProvider(new PostgreSQLSyntaxProvider());
		}
		else if (syntax.equals("SQLite"))
		{
			queryBuilder.setSyntaxProvider(new SQLiteSyntaxProvider());
		}
		else if (syntax.equals("Sybase"))
		{
			queryBuilder.setSyntaxProvider(new SybaseSyntaxProvider());
		}
	}//GEN-LAST:event_cmbSyntaxActionPerformed

	private void btnAnalyzeQueryActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnAnalyzeQueryActionPerformed
	{//GEN-HEADEREND:event_btnAnalyzeQueryActionPerformed
		analyzeQuery();
	}//GEN-LAST:event_btnAnalyzeQueryActionPerformed

	private void btnQueryStatisticsActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnQueryStatisticsActionPerformed
	{//GEN-HEADEREND:event_btnQueryStatisticsActionPerformed
		queryStatistics();
	}//GEN-LAST:event_btnQueryStatisticsActionPerformed

	private void btnAnalyzeMetadataContainerActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnAnalyzeMetadataContainerActionPerformed
	{//GEN-HEADEREND:event_btnAnalyzeMetadataContainerActionPerformed
		analyzeMetadataContainerContent();
	}//GEN-LAST:event_btnAnalyzeMetadataContainerActionPerformed

	private void btnAnalyzeWhereClauseActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_btnAnalyzeWhereClauseActionPerformed
	{//GEN-HEADEREND:event_btnAnalyzeWhereClauseActionPerformed
		analyzeWhereClause();
	}//GEN-LAST:event_btnAnalyzeWhereClauseActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAnalyzeMetadataContainer;
    private javax.swing.JButton btnAnalyzeQuery;
    private javax.swing.JButton btnAnalyzeWhereClause;
    private javax.swing.JButton btnQueryStatistics;
    private javax.swing.JComboBox cmbSyntax;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextPane textPane;
    // End of variables declaration//GEN-END:variables

	private DefaultMutableTreeNode addTreeNode(DefaultMutableTreeNode parent, String text)
	{
		DefaultMutableTreeNode result = new DefaultMutableTreeNode(text);
		parent.add(result);
		return result;
	}

	public boolean isNullOrEmpty(String s)
	{
		return (s == null || s.length() == 0);
	}
}
