package virtualobjectsandfields;

import com.adbs.ast.*;
import com.adbs.querybuilder.*;
import com.adbs.syntax.*;
import javax.swing.JOptionPane;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.logging.Level;
import java.util.logging.Logger;

public class MainFrame extends javax.swing.JFrame
{
	private static final long serialVersionUID = 1L;

	private PlainTextSQLBuilder plainTextSQLBuilder1 = new PlainTextSQLBuilder();
	private PlainTextSQLBuilder plainTextSQLBuilder2 = new PlainTextSQLBuilder();

    public MainFrame()
	{
        initComponents();

		// center window on the desktop
		setLocationRelativeTo(null);

		// set required syntax provider
		queryBuilder1.setSyntaxProvider(new MSSQLSyntaxProvider());

		//set up SQL builders:
		plainTextSQLBuilder1.setQueryBuilder(queryBuilder1);
		plainTextSQLBuilder2.setQueryBuilder(queryBuilder1);
		// Turn on the expanding of virtual objects and fields for the plainTextSQLBuilder2.
		// Text in the second text pane will contain the query text with virtual objects expanded to theirs expressions.
		plainTextSQLBuilder2.setExpandVirtualObjects(true);
		plainTextSQLBuilder2.setExpandVirtualFields(true);

		// handle the notification from plainTextSQLBuilder1 about the query text has been changed
		plainTextSQLBuilder1.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
		{
			@Override
			public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
			{
				try
				{
					jTextPane1.setText(plainTextSQLBuilder1.getSQL());
				}
				catch (QueryBuilderException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		});

		// handle the notification from plainTextSQLBuilder2 about the query text has been changed
		plainTextSQLBuilder2.addSQLUpdatedEventListener(new SQLUpdatedEventListener()
		{
			@Override
			public void sqlUpdatedEventOccurred(SQLUpdatedEvent event)
			{
				try
				{
					jTextPane2.setText(plainTextSQLBuilder2.getSQL());
				}
				catch (QueryBuilderException ex)
				{
					Logger.getLogger(MainFrame.class.getName()).log(Level.SEVERE, null, ex);
				}
			}
		});

		// Load metadata from XML file located in resources (for demonstration purposes)
		try
		{
			Class resourceClass = MainFrame.class;
			InputStream stream = resourceClass.getResourceAsStream("Northwind.xml");
			BufferedReader br = new BufferedReader(new InputStreamReader(stream));
			StringBuilder sb = new StringBuilder();
			String line = null;

			while ((line = br.readLine()) != null)
			{
				sb.append(line + "\n");
			}

			br.close();
			queryBuilder1.getMetadataContainer().loadFromXML(sb.toString());
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}

		queryBuilder1.getMetadataContainer().beginUpdate();

		try
		{
			// ===========================================================================
			// CREATE VIRTUAL DATABASE OBJECTS AND FIELDS
			// ===========================================================================

			MetadataObject o;
			MetadataField f;

			// Virtual fields for real object
			// ===========================================================================

			o = queryBuilder1.getMetadataContainer().findObjectByFullName("Northwind.dbo.Orders");
			assert (o != null);

			// first test field - simple expression
			f = o.getFields().add();
			f.setNameStr("_OrderId_plus_1");
			f.setExpression("orders.OrderId + 1");

			// second test field - correlated sub-query
			f = o.getFields().add();
			f.setNameStr("_CustomerName");
			f.setExpression("(select c.CompanyName from Customers c where c.CustomerId = orders.CustomerId)");

			// Virtual object (table) with virtual fields
			// ===========================================================================

			o = queryBuilder1.getMetadataContainer().addTable("Northwind.dbo.Orders_tbl", false);
			o.setExpression("Northwind.dbo.Orders");
			o.loadFields();

			// first test field - simple expression
			f = o.getFields().add();
			f.setNameStr("_tbl_OrderId_plus_1");
			f.setExpression("Orders_tbl.OrderId + 1");

			// second test field - correlated sub-query
			f = o.getFields().add();
			f.setNameStr("_tbl_CustomerName");
			f.setExpression("(select c.CompanyName from Customers c where c.CustomerId = Orders_tbl.CustomerId)");

			// Virtual object (sub-query) with virtual fields
			// ===========================================================================

			o = queryBuilder1.getMetadataContainer().addTable("Northwind.dbo.Orders_qry", false);
			o.setExpression("(select OrderId, CustomerId, OrderDate from Orders) as dummy_alias");
			o.loadFields();

			// first test field - simple expression
			f = o.getFields().add();
			f.setNameStr("_qry_OrderId_plus_1");
			f.setExpression("Orders_qry.OrderId + 1");

			// second test field - correlated sub-query
			f = o.getFields().add();
			f.setNameStr("_qry_CustomerName");
			f.setExpression("(select c.CompanyName from Customers c where c.CustomerId = Orders_qry.CustomerId)");


			queryBuilder1.setSQL("SELECT dummy_alias._qry_OrderId_plus_1, dummy_alias._qry_CustomerName FROM Orders_qry dummy_alias");
		}
		catch (Exception ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
		finally
		{
			queryBuilder1.getMetadataContainer().endUpdate();
		}
    }

    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane3 = new javax.swing.JScrollPane();
        jTextArea1 = new javax.swing.JTextArea();
        jSplitPane1 = new javax.swing.JSplitPane();
        queryBuilder1 = new com.adbs.querybuilder.QueryBuilder();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTextPane1 = new javax.swing.JTextPane();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTextPane2 = new javax.swing.JTextPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("VirtualObjectsAndFields Demo");

        jScrollPane3.setBorder(null);

        jTextArea1.setBackground(java.awt.SystemColor.info);
        jTextArea1.setColumns(20);
        jTextArea1.setEditable(false);
        jTextArea1.setFont(new java.awt.Font("Tahoma", 1, 11));
        jTextArea1.setForeground(new java.awt.Color(128, 0, 0));
        jTextArea1.setRows(4);
        jTextArea1.setText("This example demonstrates creation and using of virtual database objects and fields.\nSwitching the bottom tabs you can see the query text with virtual objects or the real query \nwhere the virtual objects are expanded to theirs expressions.\nSee the source code for details.");
        jTextArea1.setAutoscrolls(false);
        jTextArea1.setFocusable(false);
        jTextArea1.setMargin(new java.awt.Insets(5, 20, 5, 5));
        jScrollPane3.setViewportView(jTextArea1);

        getContentPane().add(jScrollPane3, java.awt.BorderLayout.PAGE_START);

        jSplitPane1.setDividerLocation(350);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        jSplitPane1.setResizeWeight(1.0);
        jSplitPane1.setLeftComponent(queryBuilder1);

        jTextPane1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane1FocusLost(evt);
            }
        });
        jScrollPane1.setViewportView(jTextPane1);

        jTabbedPane1.addTab("Query text with virtual objects and fields", jScrollPane1);

        jTextPane2.setBackground(new java.awt.Color(240, 240, 240));
        jTextPane2.setEditable(false);
        jTextPane2.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextPane2FocusLost(evt);
            }
        });
        jScrollPane2.setViewportView(jTextPane2);

        jTabbedPane1.addTab("Real query text with virtual objects expanded to their expressions", jScrollPane2);

        jSplitPane1.setRightComponent(jTabbedPane1);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void jTextPane1FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane1FocusLost
	{//GEN-HEADEREND:event_jTextPane1FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			queryBuilder1.setSQL(jTextPane1.getText());
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
	}//GEN-LAST:event_jTextPane1FocusLost

	private void jTextPane2FocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_jTextPane2FocusLost
	{//GEN-HEADEREND:event_jTextPane2FocusLost
		// when the focus is going out of the text pane, update the query builder with new query text
		try
		{
			if (!jTextPane2.getText().equals(plainTextSQLBuilder2.getSQL()))
			{
				queryBuilder1.setSQL(jTextPane2.getText());
			}
		}
		catch (QueryBuilderException ex)
		{
			Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, null, ex);
			JOptionPane.showMessageDialog(this, ex.getMessage());
		}
	}//GEN-LAST:event_jTextPane2FocusLost

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTextArea jTextArea1;
    private javax.swing.JTextPane jTextPane1;
    private javax.swing.JTextPane jTextPane2;
    private com.adbs.querybuilder.QueryBuilder queryBuilder1;
    // End of variables declaration//GEN-END:variables

}
